#!/bin/sh
set -e    # exit on error (except commands bracketed with "set +e ... set -e")
#
# Script to run the CHAP conversion-detection pipeline.
#

REPEATMASKER=RepeatMasker
CODE=../bin
SEQ_DIR=seq.d
EXON_DIR=annot.d
ALL_HDR=../all_gc.headers.txt
#NR_HDR=../non_redundancy.headers.txt
#OPTION1="one-to-one"                    # not used
OPTION2="one-to-many"
OPTION3="many-to-many"
CHAIN_DIR=ortho.d/ch_pair.d              # full chained pairwise alignments
OTM_DIR=ortho.d/$OPTION2-ortho.d         # "one-to-many" ortholog calls
MTM_DIR=ortho.d/$OPTION3-ortho.d         # "many-to-many" ortholog calls
TEMP=temp.d
SCRIPT=`echo $0 | sed -e 's;.*/;;'`      # actual name used, with path removed


# ---- get arguments and check tree ----

if [ $# -ne 1 ] && [ $# -ne 2 ]
then
	echo "Usage:  $SCRIPT species_tree_file [--no_rm]"
	exit 1
fi

if [ ! -d $SEQ_DIR ]
then
	echo "Sequence directory not found.  Please create $SEQ_DIR and copy sequence files into it."
	exit 1
fi

rm -rf $TEMP; mkdir -p $TEMP
ls $SEQ_DIR > $TEMP/all_species.txt
$CODE/check_species_tree $1 $TEMP/all_species.txt

# ---- make working copies of sequence files, and run RepeatMasker ----

if [ $# -eq 1 ]
then
	rm -rf new_seq.d; mkdir -p new_seq.d
	rm -rf rm_out.d;  mkdir -p rm_out.d

	for sp in `ls $SEQ_DIR`
	do
		echo ">"$sp > $TEMP/name
		sed -e '1d' $SEQ_DIR/$sp > $TEMP/$sp
		cat $TEMP/name $TEMP/$sp > new_seq.d/$sp
	done

	for sp in `ls new_seq.d`
	do
		rm -rf $TEMP; mkdir -p $TEMP
		mv new_seq.d/$sp $TEMP/$sp
		$REPEATMASKER -xsmall $TEMP/$sp
		#cp $TEMP/$sp.masked new_seq.d/$sp
		$CODE/check_mask $SEQ_DIR/$sp $TEMP/$sp.masked > new_seq.d/$sp
		cp $TEMP/$sp.out rm_out.d/$sp.out
	done
elif [ $2 = "--no_rm" ]
then
	for sp in `ls $SEQ_DIR`
	do
		if [ ! -f new_seq.d/$sp ] || [ ! -f rm_out.d/$sp.out ]
		then
			echo "Sequence $sp is not masked.  Please remove the --no_rm option."
			exit 1
		fi
	done
else
	echo "Usage:  $SCRIPT species_tree_file [--no_rm]"
	exit 1
fi

# ---- find orthologs ----

rm -rf ortho.d; mkdir -p ortho.d
mkdir ortho.d/self.d
mkdir ortho.d/pair.d
#mkdir ortho.d/ch_self.d
mkdir $CHAIN_DIR
mkdir ortho.d/data.d
#mkdir ortho.d/$OPTION1-ortho.d
mkdir $OTM_DIR
mkdir $MTM_DIR

for sp in `ls new_seq.d`
do
	$CODE/lastz T=2 Y=3400 new_seq.d/$sp --self --nomirror --format=maf > ortho.d/self.d/$sp.maf
done

for sp1 in `ls new_seq.d`
do
	for sp2 in `ls new_seq.d`
	do
		if [ $sp1 != $sp2 ]
		then
			rm -rf $TEMP; mkdir -p $TEMP
			$CODE/lastz T=2 Y=3400 new_seq.d/$sp1 new_seq.d/$sp2 --format=maf > ortho.d/pair.d/$sp1.$sp2.maf
			$CODE/cage_mask ortho.d/self.d/$sp1.maf ortho.d/pair.d/$sp1.$sp2.maf $TEMP/a.maf $TEMP/ab_new.maf 1
			$CODE/cage_mask ortho.d/self.d/$sp2.maf $TEMP/ab_new.maf $TEMP/b.maf $TEMP/new.maf 2
			if [ -f $EXON_DIR/$sp1.codex ]
			then
				$CODE/cage_chain $TEMP/a.maf $TEMP/new_a.maf rm_out.d/$sp1.out rm_out.d/$sp1.out $EXON_DIR/$sp1.codex $EXON_DIR/$sp1.codex
			else
				$CODE/cage_chain $TEMP/a.maf $TEMP/new_a.maf rm_out.d/$sp1.out rm_out.d/$sp1.out
			fi
			if [ -f $EXON_DIR/$sp2.codex ]
			then
				$CODE/cage_chain $TEMP/b.maf $TEMP/new_b.maf rm_out.d/$sp2.out rm_out.d/$sp2.out $EXON_DIR/$sp2.codex $EXON_DIR/$sp2.codex
			else
				$CODE/cage_chain $TEMP/b.maf $TEMP/new_b.maf rm_out.d/$sp2.out rm_out.d/$sp2.out
			fi
			if [ -f $EXON_DIR/$sp1.codex ] && [ -f $EXON_DIR/$sp2.codex ]
			then
				$CODE/cage_chain $TEMP/new.maf $TEMP/new_ab.maf rm_out.d/$sp1.out rm_out.d/$sp2.out $EXON_DIR/$sp1.codex $EXON_DIR/$sp2.codex
			else
				$CODE/cage_chain $TEMP/new.maf $TEMP/new_ab.maf rm_out.d/$sp1.out rm_out.d/$sp2.out
			fi
			#cp $TEMP/new_a.maf ortho.d/ch_self.d/$sp1.maf
			#cp $TEMP/new_b.maf ortho.d/ch_self.d/$sp2.maf
			cp $TEMP/new_ab.maf $CHAIN_DIR/$sp1.$sp2.maf
			cat $TEMP/new_a.maf > ortho.d/data.d/$sp1.$sp2.maf
			cat $TEMP/new_b.maf >> ortho.d/data.d/$sp1.$sp2.maf
			cat $TEMP/new_ab.maf >> ortho.d/data.d/$sp1.$sp2.maf
			echo "orthologous mappings between $sp1 and $sp2"
			#$CODE/cage_ortho ortho.d/data.d/$sp1.$sp2.maf $OPTION1 > ortho.d/$OPTION1-ortho.d/$sp1.$sp2.maf
			$CODE/cage_ortho ortho.d/data.d/$sp1.$sp2.maf $OPTION2 > $OTM_DIR/$sp1.$sp2.maf
			$CODE/cage_ortho ortho.d/data.d/$sp1.$sp2.maf $OPTION3 > $MTM_DIR/$sp1.$sp2.maf
		fi
	done
done
rm -rf ortho.d/self.d
rm -rf ortho.d/pair.d
#rm -rf ortho.d/ch_self.d           # no longer created
rm -rf ortho.d/data.d
#rm -rf ortho.d/$OPTION1-ortho.d    # no longer created

# ensure that all orthologous files are non-empty
for file in `ls $OTM_DIR`
do
	if [ ! -s $OTM_DIR/$file ]
	then
		echo "$OTM_DIR/$file is empty"
		exit 1
	fi
done
for file in `ls $MTM_DIR`
do
	if [ ! -s $MTM_DIR/$file ]
	then
		echo "$MTM_DIR/$file is empty"
		exit 1
	fi
done

# ---- get maximum descent info ----

echo "get maximum descent"
rm -rf $TEMP; mkdir -p $TEMP
for sp1 in `ls new_seq.d`
do
	echo $sp1
	rm -rf $sp1.d; mkdir -p $sp1.d
	$CODE/lastz new_seq.d/$sp1 --self Q=../human_dog.q --maf > $sp1.d/$sp1.maf
	$CODE/remove_repeats $sp1.d/$sp1.maf > $sp1.d/$sp1.remove_repeats.maf
	$CODE/remove_lower_tri $sp1.d/$sp1.remove_repeats.maf > $sp1.d/$sp1.remove_lower_tri.maf
	$CODE/chain $sp1.d/$sp1.remove_lower_tri.maf > $sp1.d/$sp1.chain
	for sp2 in `ls new_seq.d`
	do
		if [ $sp1 != $sp2 ]
		then
			echo "#$sp1.$sp2" >> $TEMP/all.md
			$CODE/md_quadruplet_entire_region $sp1.d/$sp1.chain $OTM_DIR/$sp1.$sp2.maf $MTM_DIR/$sp1.$sp2.maf >> $TEMP/all.md
			mkdir $sp1.d/$sp2.d    # needed later, for p-values
		fi
	done
	rm -f $sp1.d/$sp1.maf
	#rm -f $sp1.d/$sp1.remove_repeats.maf    # keep this for Gmaj
	rm -f $sp1.d/$sp1.remove_lower_tri.maf
	rm -f $sp1.d/$sp1.chain
done

# ---- calculate p-values and call conversions ----

echo "calculate p-values and find conversions"
if [ -f ../p_value_table.txt ]
then
	$CODE/3seq_2D_cluster_fast $TEMP/all.md ../p_value_table.txt > $TEMP/all.pvalue
else
	$CODE/3seq_2D_cluster $TEMP/all.md > $TEMP/all.pvalue
fi
$CODE/separate_pvalue $TEMP/all.pvalue    # output goes to $sp1.d/$sp2.d/all.pvalue

#echo
#echo "======================Results of Conversion=========================="
#echo "Species                 Paralogous pairs          Conversion"
for sp1 in `ls new_seq.d`
do
	parameters=""
	for sp2 in `ls new_seq.d`
	do
		if [ $sp1 != $sp2 ]
		then
			parameters="$parameters $sp1.d/$sp2.d/all.pvalue"
		fi
	done
	$CODE/FDR_cluster $parameters > $TEMP/FDR.txt
	pvalue=`$CODE/FDR_test $TEMP/FDR.txt`
	$CODE/combine_all_pvalue_cluster $parameters $pvalue $TEMP/all.corrected.pvalue
	$CODE/show_GC_cluster $TEMP/all.corrected.pvalue >> $TEMP/all.gc
	rm -f $TEMP/FDR.txt $TEMP/all.corrected.pvalue    # reused
	rm -rf $sp1.d/*.d    # includes $sp1.d/$sp2.d/all.pvalue, but need to keep $sp1.d/$sp1.remove_repeats.maf
done

# ---- remove redundancies and finalize output ----

rm -f all.gc non-redundant.gc
rm -f all.remove_redundancy.gc all_1.gc species_tree_with_index.txt conversion_in_tree.txt all.gc.tmp
$CODE/GC_history $1 $TEMP/all.gc > all.remove_redundancy.gc
	# also creates all_1.gc, species_tree_with_index.txt, conversion_in_tree.txt
cat species_tree_with_index.txt $ALL_HDR all_1.gc > all.gc.tmp
$CODE/expand_branches all.gc.tmp all.remove_redundancy.gc > all.gc
$CODE/reformat_nonredundant all.gc all.remove_redundancy.gc > non-redundant.gc
rm -f all.remove_redundancy.gc all_1.gc all.gc.tmp
#rm -f species_tree_with_index.txt    # keep this, as suggested by Federico
rm -f conversion_in_tree.txt    # misleading, as it is based only on upper-bound edges
rm -rf $TEMP

# ---- print some instructions for viewing results ----

echo
echo 'To view the conversion results with Gmaj (requires Java), run a command like this:'
echo '  ../gmaj.sh refseq_name [genomic_offset] ["title"] [exon_color]'
echo 'Examples:'
echo '  ../gmaj.sh human'
echo '  ../gmaj.sh human 31334805'
echo '  ../gmaj.sh human 31334805 "Conversions in the human CCL region"'
echo '  ../gmaj.sh human 31334805 "Conversions in the human CCL region" LightYellow'
echo '  ../gmaj.sh human 0 "" None'
echo 'Parameters are position-dependent; if you do not want an offset'
echo 'or a title, use 0 or "" respectively to reach subsequent options.'
echo 'Color list: http://globin.bx.psu.edu/dist/gmaj/gmaj_input.html#color'
echo '(or use "None" to suppress automatic underlays).  Color is only used'
echo 'if there is no pre-existing underlay file for a particular sequence.'
echo
echo 'Or, to view the ortholog calls (also with Gmaj), use:'
echo "  ../gmaj-mtm.sh refseq_name outseq_name"    # ===> update to gmaj-ortho.sh
echo 'Example:'
echo "  ../gmaj-mtm.sh human owl_monkey"           # ===> update to gmaj-ortho.sh
echo

